/**
 * @file This file contains methods for python-node to communicate between nodejs and python
 */
import * as zmq from 'zeromq';
import { Session } from '../types/python-object';
/**
 * Executor class: responsible for handling sessions and ipython and exeucting python statements.
 */
export default class Executor {
    static sessions: Session[];
    static latestIoPort: number;
    static latestShellPort: number;
    /**
     * open a session which is bound to a scope
     */
    static openSession: (scope: string) => Promise<void>;
    static closeSession: (scope: string) => Promise<void>;
    /**
     * exit a session
     */
    static exit: (scope: string) => Promise<void>;
    /**
     * create a new session
     */
    static createSession: (id: string) => Promise<Session>;
    /**
     * handle subscriber message. This protocol is used to receive stdout and stderr from ipython kernel
     */
    static handleSubscriberMsg: (socketSubscriber: zmq.Subscriber) => void;
    /**
     * handle dealer message. This protocol is used to send codes to ipython and receive execution message
     */
    static handleDealerMsg: (socketDealer: zmq.Dealer, session: Session) => void;
    /**
     * execute codes.
     * @param scope the scope name
     * @param code codes to be executed
     */
    static execute: (scope: string, code: string, user_expressions?: any) => Promise<unknown>;
}
