"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const communication_1 = __importDefault(require("../communication/communication"));
const python_helper_1 = require("./python-helper");
/**
 * get a random identifier for a PythonObject
 */
function getId() {
    return 'id_' + Date.now() + parseInt(String(Math.random() * 100000));
}
/**
 * @class main class representing a python instance.
 */
class Python {
    constructor() {
        // all python statement that will be executed in next turn
        this.statements = [];
        /**
         * used for named arguments
         */
        this.nA = (arg) => {
            return Object.assign(Object.assign({}, arg), { __pipcook__args: true });
        };
        /**
         * run shell commands, same as ! operator in ipython kernel
         */
        this.runshell = (shell) => {
            this.statements.push(`!${shell}`);
        };
        /**
         * import package
         */
        this.import = (packageName) => {
            const identifier = getId();
            this.statements.push(`import ${packageName} as ${identifier}`);
            const pythonObject = python_helper_1.getObject(identifier, this.statements);
            return pythonObject;
        };
        /**
         * install pacakge
         * @param packageName: name of package
         * @param version: version of package, default to latest
         */
        this.install = (packageName, version) => {
            this.statements.push(`!pip install ${packageName}${version ? '==' + version : ''}`);
        };
        /**
         * run some python built in function.
         * @param functionName: build-in function name
         * @param object: argument. now just support 1
         */
        this.buildin = (functionName, object) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${functionName}(${python_helper_1.conversion(object)})`);
            const pythonObject = python_helper_1.getObject(identifier, this.statements);
            return pythonObject;
        };
        /**
         * create python number
         */
        this.createNumber = (number) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${number}`);
            const pythonObject = python_helper_1.getObject(identifier, this.statements);
            return pythonObject;
        };
        /**
         * create python string
         */
        this.createString = (string, isRaw = false) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${isRaw ? 'r' : ''}'${string}'`);
            const pythonObject = python_helper_1.getObject(identifier, this.statements);
            return pythonObject;
        };
        /**
         * create python boolean type
         */
        this.createBoolean = (value) => {
            const identifier = getId();
            if (value) {
                this.statements.push(`${identifier} = True`);
            }
            else {
                this.statements.push(`${identifier} = False`);
            }
            const pythonObject = python_helper_1.getObject(identifier, this.statements);
            return pythonObject;
        };
        /**
         * create python none
         */
        this.createNone = () => {
            const identifier = getId();
            this.statements.push(`${identifier} = None`);
            const pythonObject = python_helper_1.getObject(identifier, this.statements);
            return pythonObject;
        };
        /**
         * create python tuple
         */
        this.createTuple = (value) => {
            const identifier = getId();
            const argumentElements = value.map((ele) => python_helper_1.conversion(ele));
            this.statements.push(`${identifier} = (${argumentElements.join(',')})`);
            const pythonObject = python_helper_1.getObject(identifier, this.statements);
            return pythonObject;
        };
        /**
         * create python list
         */
        this.createList = (value) => {
            const identifier = getId();
            const argumentElements = value.map((ele) => python_helper_1.conversion(ele));
            this.statements.push(`${identifier} = [${argumentElements.join(',')}]`);
            const pythonObject = python_helper_1.getObject(identifier, this.statements);
            return pythonObject;
        };
        /**
         * create python set
         */
        this.createSet = (value) => {
            const identifier = getId();
            const argumentElements = value.map((ele) => python_helper_1.conversion(ele));
            this.statements.push(`${identifier} = set((${argumentElements.join(',')}))`);
            const pythonObject = python_helper_1.getObject(identifier, this.statements);
            return pythonObject;
        };
        /**
         * create python dictionary
         */
        this.createDictionary = (value) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${JSON.stringify(value)}`);
            const pythonObject = python_helper_1.getObject(identifier, this.statements);
            return pythonObject;
        };
        /**
         * print in python
         */
        this.print = (object) => {
            this.statements.push(`print(${python_helper_1.conversion(object)})`);
        };
        /**
         * same as a == b in python
         */
        this.equal = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${python_helper_1.conversion(object1)} == ${python_helper_1.conversion(object2)}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        /**
         * same as a!=b in python
         */
        this.notEqual = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${python_helper_1.conversion(object1)} != ${python_helper_1.conversion(object2)}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        /**
         * same as a > b in python
         */
        this.larger = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${python_helper_1.conversion(object1)} > ${python_helper_1.conversion(object2)}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        /**
         * same as a < b in python
         */
        this.smaller = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${python_helper_1.conversion(object1)} < ${python_helper_1.conversion(object2)}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        /**
         * same as a >= b in python
         */
        this.largerEqual = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${python_helper_1.conversion(object1)} >= ${python_helper_1.conversion(object2)}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        /**
         * same as a <= b in python
         */
        this.smallerEqual = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${python_helper_1.conversion(object1)} <= ${python_helper_1.conversion(object2)}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        /**
         * same as a and b in python
         */
        this.and = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${python_helper_1.conversion(object1)} and ${python_helper_1.conversion(object2)}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        /**
         * same as a or b in python
         */
        this.or = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${python_helper_1.conversion(object1)} or ${python_helper_1.conversion(object2)}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        /**
         * same not a in python
         */
        this.not = (object) => {
            const identifier = getId();
            this.statements.push(`${identifier} = not ${python_helper_1.conversion(object)}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        /**
         * if statement
         * @param condition if condition
         * @param execution if body expression
         */
        this.if = (condition, execution) => {
            this.statements.push(`if ${condition.__pipcook__identifier}:`);
            const statementsTemp = this.statements;
            this.statements = [];
            execution();
            const ifStatements = this.statements.map((ele) => '\t' + ele);
            this.statements = [...statementsTemp, ...ifStatements];
        };
        // while = (condition: PythonObject, execution: Function) => {
        //   this.statements.push(`while ${condition.__pipcook__identifier}:`);
        //   const statementsTemp = this.statements;
        //   this.statements = [];
        //   execution();
        //   const ifStatements = this.statements.map((ele: string) => '\t'+ele);
        //   this.statements = [...statementsTemp, ...ifStatements];
        // }
        /**
         * same as a + b in python
         */
        this.add = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${python_helper_1.conversion(object1)} + ${python_helper_1.conversion(object2)}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        /**
         * same as a - b in python
         */
        this.minus = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${python_helper_1.conversion(object1)} - ${python_helper_1.conversion(object2)}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        /**
         * same as a * b in python
         */
        this.multiply = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${python_helper_1.conversion(object1)} * ${python_helper_1.conversion(object2)}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        /**
         * same as a / b in python
         */
        this.divide = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${python_helper_1.conversion(object1)} / ${python_helper_1.conversion(object2)}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        /**
         * same as a % b in python
         */
        this.mod = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${python_helper_1.conversion(object1)} % ${python_helper_1.conversion(object2)}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        /**
         * same as a ** b in python
         */
        this.pow = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${python_helper_1.conversion(object1)} ** ${python_helper_1.conversion(object2)}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        /**
         * same as a // b in python
         */
        this.floorDivide = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${python_helper_1.conversion(object1)} // ${python_helper_1.conversion(object2)}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        /**
         * this is for those cases where above functions does not meet user's requirements.
         * This is to run the raw python statement.
         * Please use convert method to convert PythonObject to its identifier when use this method
         */
        this.runRaw = (raw) => {
            raw = raw.replace(/\n/g, ' ');
            const identifier = getId();
            this.statements.push(`${identifier} = ${raw}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        this.evaluate = (object) => __awaiter(this, void 0, void 0, function* () {
            this.statements.push(`${object.__pipcook__identifier}`);
            const result = yield Python.constructPythonStatements(this, { type: `type(${object.__pipcook__identifier})`, value: `${object.__pipcook__identifier}` });
            try {
                const res = {
                    type: result.type,
                    value: eval(result.value)
                };
                return res;
            }
            catch (err) {
                return {
                    type: result.type,
                    value: result.value
                };
            }
        });
        this.createNumpyFromTf = (tensor) => {
            const identifier = getId();
            this.statements.push(`import numpy as np`);
            this.statements.push(`${identifier} = np.array(${JSON.stringify(tensor.arraySync())}, '${tensor.dtype}')`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        this.reconnect = () => __awaiter(this, void 0, void 0, function* () {
            yield Python.constructPythonStatements(this);
            yield communication_1.default.closeSession(this.scope);
            yield communication_1.default.openSession(this.scope);
        });
    }
    /**
     *
     * @param python Python instance
     */
    static constructPythonStatements(python, user_expressions) {
        return __awaiter(this, void 0, void 0, function* () {
            const statements = python.statements;
            let codes = '';
            statements.forEach((statement) => {
                codes += statement + '\n';
            });
            const result = yield communication_1.default.execute(python.scope, codes, user_expressions);
            python.statements.splice(0, python.statements.length);
            return result;
        });
    }
    /**
     * static method: bind python-node codes into some scope which is associated to one ipyhton kernel
     * @param scope scope name
     * @param callback python-node codes
     */
    static scope(scope, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            let python = this.pythons.find((e) => e.scope === scope);
            if (!python) {
                python = new Python();
                python.scope = scope;
                this.pythons.push(python);
            }
            yield communication_1.default.openSession(scope);
            yield callback(python);
            yield this.constructPythonStatements(python);
        });
    }
    /**
     * same as from ... import ... in python
     * @param packageName name of the package
     * @param importNames names to be imported
     */
    fromImport(packageName, importNames) {
        const objects = [];
        importNames.forEach((item) => {
            const identifier = getId();
            this.statements.push(`from ${packageName} import ${item} as ${identifier}`);
            const pythonObject = python_helper_1.getObject(identifier, this.statements);
            objects.push(pythonObject);
        });
        return objects;
    }
}
exports.default = Python;
// static parameter: all Python instances currently in the runtime
Python.pythons = [];
/** */
Python.exit = (scope) => __awaiter(void 0, void 0, void 0, function* () {
    yield communication_1.default.exit(scope);
});
/**
 * convert raw PythonObject into its identifier representation which is used in python kernel
 */
Python.convert = (object) => {
    return python_helper_1.conversion(object);
};
//# sourceMappingURL=python.js.map